/*
Author: Adam C. Migus <adam@migus.org>

Copyright (c) 2004 Adam C. Migus.  All rights reserved

This library is released under the GNU GPL and BSD software licenses.
You may choose to use one or the other, BUT NOT BOTH.  The GNU GPL
license is located in the file named COPYING.  The BSD license is located
in the file named COPYING.BSD.  Please contact us if there are any
questions.

$Id: create.c.in,v 1.7 2005/07/10 09:38:51 alcapone Exp $
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifdef HAVE_STDARG_H
#include <stdarg.h>
#endif

#include <idmef_types.h>
#include <idmef_global.h>
#include <idmef_list.h>

IDMEFPtr
newIDMEF_Message(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_DOC;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"IDMEF-Message";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newIDMEF_Message";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newIDMEF_Message_version(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"version";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newIDMEF_Message_version";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newIDMEF_Message_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newIDMEF_Message_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newIDMEF_Message_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newIDMEF_Message_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newIDMEF_Message_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newIDMEF_Message_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newIDMEF_Message_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newIDMEF_Message_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAlert(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"Alert";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAlert";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAlert_messageid(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"messageid";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAlert_messageid";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAlert_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAlert_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAlert_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAlert_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAlert_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAlert_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAlert_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAlert_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newHeartbeat(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"Heartbeat";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newHeartbeat";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newHeartbeat_messageid(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"messageid";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newHeartbeat_messageid";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newHeartbeat_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newHeartbeat_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newHeartbeat_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newHeartbeat_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newHeartbeat_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newHeartbeat_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newHeartbeat_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newHeartbeat_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newHeartbeatInterval(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"interval";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newHeartbeatInterval";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newHeartbeatInterval_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newHeartbeatInterval_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newHeartbeatInterval_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newHeartbeatInterval_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newHeartbeatInterval_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newHeartbeatInterval_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newHeartbeatInterval_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newHeartbeatInterval_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newToolAlert(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"ToolAlert";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newToolAlert";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newToolAlert_name(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"name";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newToolAlert_name";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newToolAlert_command(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"command";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newToolAlert_command";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newToolAlert_alertident(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"alertident";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newToolAlert_alertident";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newToolAlert_analyzerid(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"analyzerid";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newToolAlert_analyzerid";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newToolAlert_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newToolAlert_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newToolAlert_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newToolAlert_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newToolAlert_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newToolAlert_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newToolAlert_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newToolAlert_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newCorrelationAlert(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"CorrelationAlert";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newCorrelationAlert";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newCorrelationAlert_name(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"name";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newCorrelationAlert_name";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newCorrelationAlert_alertident(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"alertident";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newCorrelationAlert_alertident";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newCorrelationAlert_analyzerid(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"analyzerid";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newCorrelationAlert_analyzerid";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newCorrelationAlert_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newCorrelationAlert_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newCorrelationAlert_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newCorrelationAlert_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newCorrelationAlert_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newCorrelationAlert_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newCorrelationAlert_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newCorrelationAlert_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newOverflowAlert(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"OverflowAlert";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newOverflowAlert";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newOverflowAlert_program(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"program";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newOverflowAlert_program";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newOverflowAlert_size(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"size";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newOverflowAlert_size";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newOverflowAlert_buffer(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"buffer";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newOverflowAlert_buffer";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newOverflowAlert_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newOverflowAlert_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newOverflowAlert_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newOverflowAlert_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newOverflowAlert_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newOverflowAlert_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newOverflowAlert_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newOverflowAlert_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAnalyzer(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"Analyzer";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAnalyzer";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAnalyzer_analyzerid(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"analyzerid";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAnalyzer_analyzerid";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAnalyzer_name(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"name";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAnalyzer_name";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAnalyzer_manufacturer(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"manufacturer";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAnalyzer_manufacturer";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAnalyzer_model(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"model";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAnalyzer_model";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAnalyzer_version(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"version";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAnalyzer_version";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAnalyzer_class(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"class";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAnalyzer_class";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAnalyzer_ostype(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"ostype";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAnalyzer_ostype";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAnalyzer_osversion(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"osversion";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAnalyzer_osversion";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAnalyzer_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAnalyzer_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAnalyzer_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAnalyzer_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAnalyzer_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAnalyzer_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAnalyzer_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAnalyzer_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newClassification(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"Classification";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newClassification";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newClassification_ident(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"ident";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newClassification_ident";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newClassification_text(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"text";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newClassification_text";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newReference(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"Reference";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newReference";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newReference_origin(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"origin";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newReference_origin";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newReference_meaning(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"meaning";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newReference_meaning";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newReference_name(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"name";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newReference_name";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newReference_url(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"url";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newReference_url";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newSource(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"Source";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newSource";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newSource_ident(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"ident";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newSource_ident";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newSource_spoofed(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"spoofed";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newSource_spoofed";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newSource_interface(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"interface";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newSource_interface";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newSource_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newSource_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newSource_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newSource_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newSource_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newSource_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newSource_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newSource_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newTarget(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"Target";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newTarget";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newTarget_ident(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"ident";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newTarget_ident";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newTarget_decoy(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"decoy";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newTarget_decoy";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newTarget_interface(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"interface";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newTarget_interface";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newTarget_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newTarget_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newTarget_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newTarget_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newTarget_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newTarget_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newTarget_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newTarget_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAssessment(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"Assessment";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAssessment";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAssessmen_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAssessmen_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAssessmen_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAssessmen_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAssessmen_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAssessmen_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAssessmen_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAssessmen_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAdditionalData(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"AdditionalData";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAdditionalData";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAdditionalData_type(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"type";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAdditionalData_type";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAdditionalData_meaning(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"meaning";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAdditionalData_meaning";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAdditionalData_boolean(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"boolean";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAdditionalData_boolean";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAdditionalData_byte(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"byte";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAdditionalData_byte";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAdditionalData_character(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"character";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAdditionalData_character";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAdditionalData_date_time(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"date_time";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAdditionalData_date_time";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAdditionalData_integer(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"integer";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAdditionalData_integer";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAdditionalData_ntpstamp(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"ntpstamp";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAdditionalData_ntpstamp";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAdditionalData_portlist(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"portlist";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAdditionalData_portlist";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAdditionalData_real(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"real";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAdditionalData_real";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAdditionalData_string(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"string";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAdditionalData_string";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAdditionalData_byte_string(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"byte_string";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAdditionalData_byte_string";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAdditionalData_xmltext(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmltext";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAdditionalData_xmltext";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAdditionalData_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAdditionalData_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAdditionalData_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAdditionalData_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAdditionalData_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAdditionalData_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAdditionalData_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAdditionalData_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newCreateTime(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"CreateTime";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newCreateTime";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newCreateTime_ntpstamp(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"ntpstamp";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newCreateTime_ntpstamp";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newCreateTime_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newCreateTime_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newCreateTime_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newCreateTime_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newCreateTime_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newCreateTime_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newCreateTime_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newCreateTime_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAnalyzerTime(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"AnalyzerTime";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAnalyzerTime";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAnalyzerTime_ntpstamp(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"ntpstamp";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAnalyzerTime_ntpstamp";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAnalyzerTime_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAnalyzerTime_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAnalyzerTime_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAnalyzerTime_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAnalyzerTime_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAnalyzerTime_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAnalyzerTime_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAnalyzerTime_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newDetectTime(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"DetectTime";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newDetectTime";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newDetectTime_ntpstamp(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"ntpstamp";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newDetectTime_ntpstamp";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newDetectTim_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newDetectTim_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newDetectTim_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newDetectTim_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newDetectTim_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newDetectTim_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newDetectTim_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newDetectTim_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newImpact(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"Impact";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newImpact";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newImpact_severity(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"severity";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newImpact_severity";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newImpact_completion(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"completion";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newImpact_completion";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newImpact_type(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"type";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newImpact_type";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newImpact_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newImpact_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newImpact_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newImpact_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newImpact_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newImpact_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newImpact_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newImpact_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAction(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"Action";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAction";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAction_category(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"category";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAction_category";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAction_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAction_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAction_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAction_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAction_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAction_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAction_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAction_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newConfidence(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"Confidence";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newConfidence";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newConfidence_rating(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"rating";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newConfidence_rating";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newConfidence_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newConfidence_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newConfidence_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newConfidence_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newConfidence_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newConfidence_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newConfidence_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newConfidence_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newNode(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"Node";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newNode";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newNode_ident(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"ident";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newNode_ident";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newNode_category(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"category";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newNode_category";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newNode_location(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"location";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newNode_location";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newNode_name(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"name";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newNode_name";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newNode_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newNode_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newNode_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newNode_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newNode_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newNode_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newNode_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newNode_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAddress(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"Address";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAddress";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAddress_ident(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"ident";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAddress_ident";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAddress_category(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"category";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAddress_category";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAddress_vlan_name(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"vlan-name";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAddress_vlan_name";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAddress_vlan_num(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"vlan-num";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAddress_vlan_num";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAddress_address(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"address";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAddress_address";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAddress_netmask(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"netmask";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAddress_netmask";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAddress_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAddress_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAddress_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAddress_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAddress_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAddress_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newAddress_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newAddress_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newUser(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"User";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newUser";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newUser_ident(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"ident";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newUser_ident";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newUser_category(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"category";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newUser_category";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newUser_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newUser_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newUser_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newUser_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newUser_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newUser_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newUser_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newUser_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newUserId(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"UserId";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newUserId";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newUserId_ident(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"ident";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newUserId_ident";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newUserId_type(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"type";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newUserId_type";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newUserId_tty(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"tty";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newUserId_tty";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newUserId_name(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"name";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newUserId_name";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newUserId_number(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"number";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newUserId_number";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newUserId_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newUserId_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newUserId_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newUserId_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newUserId_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newUserId_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newUserId_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newUserId_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newProcess(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"Process";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newProcess";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newProcess_ident(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"ident";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newProcess_ident";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newProcess_name(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"name";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newProcess_name";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newProcess_pid(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"pid";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newProcess_pid";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newProcess_path(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"path";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newProcess_path";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newProcess_arg(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"arg";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newProcess_arg";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newProcess_env(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"env";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newProcess_env";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newProcess_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newProcess_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newProcess_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newProcess_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newProcess_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newProcess_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newProcess_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newProcess_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newService(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"Service";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newService";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newService_ident(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"ident";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newService_ident";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newService_ip_version(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"ip_version";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newService_ip_version";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newService_iana_protocol_number(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"iana_protocol_number";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newService_iana_protocol_number";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newService_iana_protocol_name(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"iana_protocol_name";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newService_iana_protocol_name";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newService_name(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"name";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newService_name";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newService_port(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"port";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newService_port";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newService_portlist(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"portlist";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newService_portlist";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newService_protocol(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"protocol";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newService_protocol";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newService_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newService_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newService_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newService_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newService_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newService_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newService_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newService_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newWebService(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"WebService";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newWebService";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newWebService_url(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"url";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newWebService_url";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newWebService_cgi(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"cgi";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newWebService_cgi";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newWebService_http_method(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"http-method";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newWebService_http_method";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newWebService_arg(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"arg";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newWebService_arg";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newWebService_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newWebService_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newWebService_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newWebService_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newWebService_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newWebService_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newWebService_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newWebService_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newSNMPService(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"SNMPService";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newSNMPService";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newSNMPService_oid(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"oid";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newSNMPService_oid";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newSNMPService_securityName(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"securityName";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newSNMPService_securityName";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newSNMPService_contextName(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"contextName";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newSNMPService_contextName";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newSNMPService_contextEngineID(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"contextEngineID";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newSNMPService_contextEngineID";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newSNMPService_community(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"community";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newSNMPService_community";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newSNMPService_command(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"command";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newSNMPService_command";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newSNMPService_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newSNMPService_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newSNMPService_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newSNMPService_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newSNMPService_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newSNMPService_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newSNMPService_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newSNMPService_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFile(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"File";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFile";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFile_ident(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"ident";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFile_ident";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFile_category(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"category";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFile_category";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFile_fstype(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"fstype";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFile_fstype";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFile_file_type(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"file-type";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFile_file_type";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFile_name(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"name";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFile_name";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFile_path(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"path";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFile_path";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFile_create_time(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"create-time";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFile_create_time";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFile_modify_time(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"modify-time";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFile_modify_time";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFile_access_time(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"access-time";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFile_access_time";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFile_data_size(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"data-size";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFile_data_size";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFile_disk_size(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"disk-size";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFile_disk_size";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFile_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFile_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFile_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFile_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFile_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFile_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFile_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFile_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFileAccess(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"FileAccess";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFileAccess";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFileAccess_permission(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"permission";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFileAccess_permission";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFileAccess_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFileAccess_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFileAccess_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFileAccess_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFileAccess_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFileAccess_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newFileAccess_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newFileAccess_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newLinkage(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"Linkage";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newLinkage";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newLinkage_category(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"category";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newLinkage_category";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newLinkage_name(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"name";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newLinkage_name";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newLinkage_path(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"path";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newLinkage_path";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newLinkage_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newLinkage_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newLinkage_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newLinkage_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newLinkage_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newLinkage_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newLinkage_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newLinkage_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newInode(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"Inode";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newInode";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newInode_change_time(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"change-time";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newInode_change_time";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newInode_number(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"number";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newInode_number";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newInode_major_device(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"major-device";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newInode_major_device";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newInode_minor_device(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"minor-device";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newInode_minor_device";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newInode_c_major_device(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"c-major-device";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newInode_c_major_device";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newInode_c_minor_device(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"c-minor-device";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newInode_c_minor_device";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newInode_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newInode_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newInode_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newInode_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newInode_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newInode_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newInode_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newInode_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newChecksum(IDMEFPtr first, ...) {
	IDMEF_Type type = IDMEF_Type_NODE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"Checksum";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newChecksum";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newChecksum_algorithm(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"algorithm";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newChecksum_algorithm";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newChecksum_value(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"value";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newChecksum_value";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newChecksum_key(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_CONTENT;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"key";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newChecksum_key";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newChecksum_xmlns(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newChecksum_xmlns";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newChecksum_xmlns_idmef(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xmlns_idmef";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newChecksum_xmlns_idmef";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newChecksum_xml_lang(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_lang";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newChecksum_xml_lang";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}

IDMEFPtr
newChecksum_xml_space(const char* first, ...) {
	IDMEF_Type type = IDMEF_Type_ATTRIBUTE;
	IDMEFPtr idmefp, tidmefp;
	xmlChar *name = (xmlChar*)"xml_space";
	xmlChar *content = (xmlChar *)first;
	char *f_name = "newChecksum_xml_space";
	va_list ap;

	/* added possibility of creation of empty NODE */
        if (first == NULL && type != IDMEF_Type_NODE &&
        		     type != IDMEF_Type_CONTENT) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: missing required argument\n",
			f_name);
#endif
		return (NULL);
	}

		
	if ((idmefp = libidmef_idmefp_create(type, name)) == NULL) {
#ifdef IDMEF_DEBUG
		fprintf(stderr, "%s: error: idmefp_create(%d, %s)\n",
			f_name, type, name);
#endif
		return (NULL);
	}

	va_start(ap, first);

	tidmefp = (IDMEFPtr)first;
        
	/*
	 * Once we start processing va_args we have to assume that the
	 * user was kind enough to provide enough arguments.  If not
	 * the results will be random strings.
	 */

	switch (type) {
		case IDMEF_Type_DOC:
		case IDMEF_Type_NODE:
			break;
		case IDMEF_Type_CONTENT:
                	/* check for at least one arg due to additional
                           possibility to create empty content idmefp */
                        if (!tidmefp)
                          goto out;
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlNodeSetContent(idmefp->IDMEFXmlNodePtr, content);
			/*
			 * We loop through args for content to allow for
			 * content nodes with attributes.
			 */
			tidmefp = va_arg(ap, IDMEFPtr);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * We have to assume we're getting two arguments.
			 */
  			xmlSetProp(idmefp->IDMEFXmlNodePtr, name, content);
			/*
			 * We don't loop through args for attributes.
			 */
			goto out;
	}

	/*
	 * Process the arguments by type.
	 */
	for (; tidmefp != NULL; tidmefp = va_arg(ap, IDMEFPtr)) {
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_LOCK);
		switch (tidmefp->IDMEF_type) {
                case IDMEF_Type_DOC:
			break;
		case IDMEF_Type_NODE:
		case IDMEF_Type_CONTENT:
			/*
			 * Add this node as a child.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlAddChild(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
			break;
		case IDMEF_Type_ATTRIBUTE:
			/*
			 * Add an attribute using the attribute
			 * contained in this node.
			 */
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_LOCK);
			xmlSetProp(idmefp->IDMEFXmlNodePtr,
				tidmefp->IDMEFXmlNodePtr->properties->name,
				tidmefp->IDMEFXmlNodePtr->properties->children->content);
                        libidmef_list_insert (idmefp, tidmefp);
			libidmef_idmefp_lock_op(idmefp,
				IDMEF_Lock_Op_WRITE_UNLOCK);
//			xmlFreeNode(tidmefp->IDMEFXmlNodePtr);
			break;
		}
		libidmef_idmefp_lock_op(tidmefp, IDMEF_Lock_Op_READ_UNLOCK);
  	}

out:
	va_end(ap);
	return (idmefp);
}


